/**
 * Program: BuildHistogram.js
 * --------------------------
 * This program reads a series of midterm scores
 * from a file and publishes a histogram to the console
 * window.  The scores are grouped 0-9, 10-19, etc,
 * and the final bucket is 90-100.
 */

function BuildHistogram() {
	let callback = function(text) {
		let lines = JSFileChooser.convertToLineArray(text);
		let histogram = createArray(10, 0);
		readScores(histogram, lines);
		displayHistogram(histogram);
	}
	JSFileChooser.chooseTextFile(callback);
}

/**
 * Function: readScores
 * --------------------
 * Assumes an empty histogram array of length 10 (with all zeroes),
 * and updates the histogram to reflect the frequency of numbers
 * present in the lines array.
 */
function readScores(histogram, lines) {
	for (let i = 0; i < lines.length; i++) {
		let score = parseInt(lines[i]);
		if (score === 100) score = 99;
		let bucket = Math.floor(score/10);
		histogram[bucket]++;
	}
}

/**
 * Function: displayHistogram
 * --------------------------
 * Assumes a fully compiled histogram of ten buckets has
 * already been constructed and passed in, and printed
 * that histogram as ten lines (where each line is preceded
 * by a header that's clear what range of numbers are represented
 * by that line.)
 */
function displayHistogram(histogram) {
	let headers = createHeadersArray();
	for (let bucket = 0; bucket < histogram.length; bucket++) {
		let line = constructLine(histogram[bucket], "*");
		console.log(headers[bucket] + ": " + line);
	}
}

/**
 * Function: constructLine
 * -----------------------
 * Generates a string of the specified length, where
 * each and every character is the character specified via ch.
 */
function constructLine(length, ch) {
	let str = "";
	for (let i = 0; i < length; i++) {
		str += ch;
	}
	return str;
}

/**
 * Function: createArray
 * ---------------------
 * Creates and returns an array with exactly
 * n copies of the provided value.
 */
function createArray(n, value) {
	let array = [];
	for (let i = 0; i < n; i++) {
		array.push(value);
	}
	return array;
}

/**
 * Function: createHeadersArray
 * ----------------------------
 * Creates an array of headers--all of equal width--
 * so that each line of our histogram can be clear what
 * scores are tabulated by that line.
 */
function createHeadersArray() {
	let headers = ["   0-9"];
	for (let bucket = 1; bucket < 9; bucket++) {
		let header = " " + bucket + "0-" + bucket + "9";
		headers.push(header);
	}
	headers.push("90-100");
	return headers;
}


